@@ -55,6 +55,7 @@ class Teacher(cfg.ConfigUtils, QstatusDefs):
         self.m_lessonfile = None
         self.m_question = None
         self.m_custom_mode = False
+        self.m_lessonfile_defs = {'yes': True, 'no': False}
     def maybe_auto_new_question(self):
         if self.get_bool('new_question_automatically'):
             if self.m_timeout_handle is None:
@@ -88,17 +89,8 @@ class Teacher(cfg.ConfigUtils, QstatusDefs):
         if not self.m_lessonfile:
             self.m_P = None
             return
-        self.m_P = self.lessonfileclass()
-        try:
-            self.m_P.parse_file(self.m_lessonfile)
-        # We don't have to check for LessonfileExceptions here because
-        # the global exception hook will catch them.
-        except IOError, e:
-            self.m_P = None
-            solfege.win.display_error_message(
-"""There was an IOError while trying to parse a lesson file:
-%s.""" % e)
-            return
+        self.m_P = self.lessonfileclass(self.m_lessonfile_defs)
+        self.m_P.parse_file(self.m_lessonfile)
         if [q for q in self.m_P.m_questions if isinstance(q.get('music'), lessonfile.Cmdline)]:
             run = gu.dialog_yesno(_("The lessonfile contain potentially dangerous code because it run external programs. Run anyway?"))
             if not run:
@@ -406,6 +398,9 @@ class Gui(gtk.VBox, cfg.ConfigUtils, QstatusDefs):
         hbox.pack_start(self.g_deprecated_label)
         self.practise_box.pack_start(hbox, False, False)
         self.practise_box.reorder_child(hbox, 0)
+    def set_deprecation_text(self, oldmodule, newmodule, filename):
+        self.g_deprecated_label.set_line_wrap(True)
+        self.g_deprecated_label.set_markup("<b>The %(oldmodule)s module is deprecated. Convert \"%(filename)s\" to use the %(newmodule)s module.</b>" % locals())
     def std_buttons_add(self, *buttons):
         """
         buttons is a sequence of tuples ('buttonname', callback)
@@ -447,6 +442,7 @@ class Gui(gtk.VBox, cfg.ConfigUtils, QstatusDefs):
         if [n for n in self._std_buttons if n.split("-")[0] == 'new']:
             self.g_new.set_sensitive(True)
         for b in ('repeat', 'repeat_slowly', 'repeat_arpeggio',
+            'repeat_first', 'repeat_last',
             'repeat_melodic', 'guess_answer', 'play_tonic', 'play_music',
             'play_answer',
             'display_music', 'show', 'give_up'):
@@ -489,6 +485,7 @@ class Gui(gtk.VBox, cfg.ConfigUtils, QstatusDefs):
             self.g_new.set_sensitive(
                 not self.get_bool('config/picky_on_new_question'))
         for s in ('repeat', 'repeat_slowly', 'repeat_arpeggio',
+                'repeat_first', 'repeat_last',
                 'repeat_melodic', 'guess_answer', 'play_music',
                 'display_music', 'play_answer'):
             if s in self._std_buttons:
@@ -496,17 +493,16 @@ class Gui(gtk.VBox, cfg.ConfigUtils, QstatusDefs):
         if 'play_tonic' in self._std_buttons:
             self.g_play_tonic.set_sensitive(
                 'tonic' in self.m_t.m_P.get_question())
-        if 'show' in self._std_buttons:
-            self.g_show.set_sensitive(False)
-        if 'give_up' in self._std_buttons:
-            self.g_give_up.set_sensitive(False)
+        for n in 'show', 'give_up', 'guess_answer':
+            if n in self._std_buttons:
+                getattr(self, 'g_%s' % n).set_sensitive(False)
     def std_buttons_end_practise(self):
         # We check for the lesson parser because it is possible that a
         # completely broken lesson file was started, and the user tries
         # to exit that exercise or quit the program. And if the lesson file
         # does not have a header block, then we cannot call
         # std_buttons_start_practise
-        if self.m_t.m_P and getattr(self.m_t.m_P, 'header', None):
+        if self.m_t.m_P and hasattr(self.m_t.m_P, 'header'):
             self.std_buttons_start_practise()
         else:
             for name in self._std_buttons:
@@ -579,8 +575,16 @@ class Gui(gtk.VBox, cfg.ConfigUtils, QstatusDefs):
         """
         self.handle_config_box_visibility()
         self.handle_statistics_page_sensibility()
-        if getattr(self, 'g_flashbar', None):
+        if hasattr(self, 'g_flashbar'):
             self.g_flashbar.require_size(["|Xy"])
+        if self.m_t.m_P.header.lesson_heading:
+            self.set_lesson_heading(self.m_t.m_P.header.lesson_heading)
+        else:
+            self.set_lesson_heading(getattr(self, "lesson_heading", None))
+        if self.m_t.m_P.header.lesson_description:
+            self.set_lesson_description(self.m_t.m_P.header.lesson_description)
+        else:
+            self.set_lesson_description(getattr(self, "lesson_description", None))
     def on_end_practise(self):
         pass
     def handle_config_box_visibility(self):
